from __future__ import absolute_import
from __future__ import print_function

import random
import numpy as np
import os
import sys
import pandas
import matplotlib.pyplot as plt
import pickle
import time
from imports.simulation_core import *
from dane.createTrips import  *

# we need to import python modules from the $SUMO_HOME/tools directory
if 'SUMO_HOME' in os.environ:
    tools = os.path.join(os.environ['SUMO_HOME'], 'tools')
    sys.path.append(tools)
else:
    sys.exit("please declare environment variable 'SUMO_HOME'")

from sumolib import checkBinary  # noqa
import traci  # noqa

# Położenie folderu do którego zapisywane będa wyniki symulacji
TRACKING_OUTPUT_FOLDER = 'data_tracking'

# Procedura uruchamiania samej symulacji
def run(traffic_ctrl_system_on):

    # Run with or without GUI - uncomment a selected row
    sumoBinary = checkBinary('sumo-gui')
    # sumoBinary = checkBinary('sumo')
    
    # first, generate the route file for this simulation

    # this is the normal way of using traci. sumo is started as a
    # subprocess and then the python script connects and runs
    traci.start([sumoBinary, "-c", "dane/osm.sumocfg",
                             "--tripinfo-output", "tripinfo.xml"])
                             
    """execute the TraCI control loop"""
    ###########################################        
    
    # Część naszego kodu słuzy do śledzenia zachowania monitorów wykrywających pojazdy 
    # i śledzących stan pasa jezdni 
    dm_06 = DetectorMonitor('det06', traffic_ctrl_system_on)
    dm_04 = DetectorMonitor('det02', False)
    lm_04 = LaneMonitor('gneE4_0')
    
    # Sama symulacja odbywa się w pętli - warunek pętli
    # sprawdza czy są jeszcze jakieś pojazdy, o których SUMO wie
    # że powinny opuścić symulowany region
    step = 0
    while traci.simulation.getMinExpectedNumber() > 0:
        traci.simulationStep()        
        dm_06.tick()
        dm_04.tick()
        lm_04.tick()   
        step += 1
    
    # Struktura danych przechowująca wyniki
    set_of_results = {}
    set_of_results.update({'dm_06':dm_06.tracked_stats()})
    set_of_results.update({'dm_02':dm_04.tracked_stats()})
    set_of_results.update({'lm_04':lm_04.tracked_stats()})
    
    # Operacje kończące i zwrócenie wyników z funkcji
    traci.close()
    sys.stdout.flush()
    return set_of_results

# this is the main entry point of this script
if __name__ == "__main__":
    NO_REPETITIONS = 40 # Symulacja jest napisana tak, by od razu można było kilkukrotnie powtórzyć eksperyment
    
    # Tutaj regulowana jest liczba pojazdów na godzinę, plik trips jest generowany od nowa 
    # przy każdym uruchomieniu skryptu za pomocą specjalnej funkcji
    generate_trips_for_intensity(400) 
    
    # Za kazdym razem symulacja uruchamiana jest dwukrotnie - raz z włączonym sterowaniem ruchem za pomocą modelu (flaga traffic_ctrl_system_on)
    # a raz z wyłączonym - w celu porównania
    for i in range(NO_REPETITIONS):
        master_output_structure = {}
        master_output_structure.update({'with_model':run(True)})
        master_output_structure.update({'no_model':run(False)})
        
        # Każdy wynik zapisywany jest we wskazanym folderze w postaci pliku pickle
        timestamp = time.time()
        with open('data_tracking/result_%s.pickle'%timestamp,'wb') as f:
            pickle.dump(master_output_structure, f)
    
