'''Read data from detector logs.'''

import xml.etree.ElementTree as ET
import os
import csv


def parse_log(filename):
    tree = ET.parse(filename)
    root = tree.getroot()
    if root.tag != 'detector':
        return None
    data = {}
    for interval in root:
        att = interval.attrib
        begin = float(att['begin'])
        nveh = int(att['nVehContrib'])
        speed = round(3.6 * float(att['speed']), 3) if nveh > 0 else -1
        data[begin] = {'nveh': nveh, 'speed': speed}
    return data


def create_csv(csv_number, csv_speed, path=None):
    # read all XML files in the given path
    files = (f for f in os.listdir(path) if f.endswith('.xml'))
    data = {}
    for f in files:
        name = f.rsplit('.', 1)[0]
        detector_data = parse_log(f)
        if detector_data is not None:
            data[name] = detector_data
    # construct CSVs
    detectors = list(data.keys())
    times = list(data[detectors[0]].keys())
    # CSV for vehicles number
    with open(csv_number, 'w', newline='') as fout1:
        writer1 = csv.writer(fout1)
        writer1.writerow(['Time'] + detectors)
        for t in times:
            writer1.writerow([t] + [data[d][t]['nveh'] for d in detectors])
    # CSV for vehicles speed
    with open(csv_speed, 'w', newline='') as fout2:
        writer2 = csv.writer(fout2)
        writer2.writerow(['Time'] + detectors)
        for t in times:
            writer2.writerow([t] + [data[d][t]['speed'] for d in detectors])


if __name__ == "__main__":
    # data = parse_log('det01.xml')
    # print(data)
    create_csv('detectors-number.csv', 'detectors-speed.csv')
